
# KiCADを用いた回路・基板設計マニュアル

# 新規コンポーネント作成

`hardware/components/Tempelate/`を同階層にコピーし，
ディレクトリおよび回路図・基板ファイルの名前をコンポート名に変更する．

# コンポーネントの回路設計

## 部品の配置

使用する部品を選び，配置する．

KiCADのライブラリにない部品を使いたい場合は，ネットで探すか，自作する．
自作したシンボルは`hardware/library/WOBCLibrary.kicad_sym`に，
フットプリントは`hardware/libirary/WOBCLibrary.pretty`に追加しておく．

トランジスタのように一般的な部品は，特定の型番のものではなく一般的な記号のものを使う．

## 電源と配線

部品同士を配線する．
電源に関しては以下のシンボルを使う．

- VDD: 3.3V系の電源．IC類の電源は基本的にこれを用いる．
- GND
- VBUS: USBコネクタから供給される5V電源．
- VPP: 電池の出力そのまま等，VDDの上流の電源．
- +BATT: RTCやGNSS用のバックアップ電源．

ICの電源ピンには100nF程度のバイパスコンデンサを付ける．

## 入出力ラベルの設定

電源以外に外部と接続したいピンは，階層ラベルをつける．
階層ラベル名のprefixにコンポーネント名をつける必要はない．

## シンボルのアノテーション

Symbol reference designatorを使用し，
各シンボルについている番号（`R0`，`Q1`など）を振り直す．
モジュール作成時にインポートする作業のため，番号は100から始まるようにする．

![Annotate](/docs/image/kicad/annotate.png)


## フットプリントの割り当て

Footprint assignment toolを使用し，シンボルにフットプリントを割り当てる．

特によく使う部品のフットプリントに関しては，

| カテゴリ |      |
|----|-------|
| 抵抗 | 信号用は 0402(1005) サイズを用いる．|
| 積層セラミックコンデンサ | 1uF以下は0402(1005) サイズ，大容量の電源用には0603(1608)や0805(2012)サイズを目安に用いる．|


![Footprint Asignment](/docs/image/kicad/footprint_assignment.png)

## LCSC部品番号の割り当て

PCBAサービスで実装したい部品について，
[JLCPCB Parts Lib](https://jlcpcb.com/parts)で探した部品の番号
（JLCPCB Parts # とある，Cから始まる番号）をシンボルに設定する．
シンボルに`LCSC`というフィールドを追加し，部品番号を入力する．

![LCSC Parts Number](/docs/image/kicad/lcsc_number.png)

## Electorical Rule Check の実行

ELCを実行し，回路に間違いがないか確認する．
使っていないピンには no-conenction-flag (バツ印)を設定して，
エラーが出ないようにしておく．


# コンポーネントの基板設計

通常はモジュールごとに基板設計を行うが，コンポーネントの回路が複雑で部品点数が多いなどの
理由で基板設計データも再利用したいときには，コンポーネントごとに基板設計を行ってもよい．

注意点等はモジュールの基板設計の項目を参照．

# 新規モジュール作成

`hardware/modules/Tempelate/`を同階層にコピーし，
ディレクトリおよび回路図・基板ファイルの名前をモジュール名に変更する．

# モジュールの回路設計

## コンポーネントの挿入

各コンポーネントは，モジュール回路図内の階層シートからリンクして使う．

シートを追加し，`Sheetfile`に使いたいコンポーネントのパスを設定する．

![Sheet Properties](/docs/image/kicad/sheet_properties.png)

コンポーネントごとに基板設計を行っている場合は，以下の手順で基板設計データをコピーする．

1. モジュールプロジェクトを開く
2. 上述の通り，回路エディタで階層シートを追加しコンポーネントの回路図をリンクする．
3. 階層シート内に移動し，シンボルのアノテーションを行い部品に100から始まる番号を振り直す（階層シート内の部品のみ）
4. コンポーネントプロジェクトを開く
5. コンポーネントの基板エディタで部品と配線を全選択→コピー
6. モジュールプロジェクトを開く
7. コンポーネントの部品と配線をモジュールの基板ファイル内にペーストする
8. ペーストした基板の部品と回路図を結びつけるため，基板エディタのUpdate PCB from Schematic ツールで，Re-link ... on their reference designators にチェックを入れて Update PCB
9. 回路エディタで，シンボルのアノテーションを行い，モジュールの全ての部品について0から始まる部品を振り直す
10. 基板エディタで振り直した番号を反映させるため，Re-link ... on their reference designatorsのチェックを外してUpdate PCB
11. さらにモジュールにコンポーネントを追加する場合，1~10の手順を繰り返す．

![Update PCB](/docs/image/kicad/update_pcb.png)

## 配線

シートを右クリックして`Import Sheet Pin`で階層ラベルに設定した入出力ピンを表示し，
コンポーネント間を配線する．

# モジュールの基板設計

[ノイズ対策.COM](https://www.noise-counterplan.com)などを参考に.

## ベース基板の決定

モジュールは基本的に，60mm x 60mm x 1mm のサイズを基準としたベース基板上に実装する．
WOBCLibrary内の`BaseBoard`を用いる．

## 部品の配置

PCBAサービスで実装する部品は必ず，手実装する部品もなるべく片面に配置する．

## 配線

[JLCPCBの寸法ルール](https://jlcpcb.com/capabilities/pcb-capabilities)
を破らないように，配線幅やクリアランスをとる．

- 基板厚さ：基本的に1mmとする．
- ビア：基本的に径0.5mm，ドリル径0.3mm
- 信号線：0.2mm 以上
- 電源線：0.4mm, 0.8mm などなるべく太く．1Aで1mmが目安
- 無線信号：曲線で配線幅 1.5mm〜2.0mm，裏面にGNDプレーンを配置して50Ωのマイクロストリップ線路にする
- USB信号：配線幅 0.8mm，配線間隔 0.2mm，裏面にGNDプレーンを配置して90Ωの差動マイクロストリップ線路にする（配線が長い場合）
- 無線信号：曲線で配線幅 1.5mm〜2.0mm，裏面にGNDプレーンを配置して50Ωのマイクロストリップ線路にする
- CAN信号：配線幅 0.4mm，配線間隔 0.2mm，裏面にGNDプレーンを配置して120Ωの差動マイクロストリップ線路にする


## シルクの設定

コネクタやLEDなどが何を表すかわかるように，基板上に記しておく．

部品番号については，手で実装しない部品で書くスペースがない場合は消してもよい．

部品や他の文字，ビアと被らないように配置する．
文字サイズは最低0.8mm以上，できれば1mm以上を推奨．

署名やロゴマークなどを入れてもよい．

## ベタGNDの設定

なるべくスタブがないようにする．
外周やスタブになるところにはビアを配置する．

## Design Rule Check の実行

DLCを実行し，未配線やクリアランス違反がないか確認する．

## ガーバーファイルの書き出し

KiCADプラグインのJLCPCB Toolsを用いてガーバーファイルを書き出す．
